@@ -1,5 +1,5 @@
 /* Tarlz - Archiver with multimember lzip compression
-   Copyright (C) 2013-2023 Antonio Diaz Diaz.
+   Copyright (C) 2013-2024 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -76,7 +76,7 @@ void xLZ_decompress_write( LZ_Decoder * const decoder,
 Archive_descriptor::Archive_descriptor( const std::string & archive_name )
   : name( archive_name ), namep( name.empty() ? "(stdin)" : name.c_str() ),
     infd( non_tty_infd( archive_name, namep ) ),
-    lzip_index( infd, true, false ),
+    lzip_index( infd ),
     seekable( lseek( infd, 0, SEEK_SET ) == 0 ),
     indexed( seekable && lzip_index.retval() == 0 ) {}
 
