@@ -1,5 +1,5 @@
 /* Tarlz - Archiver with multimember lzip compression
-   Copyright (C) 2013-2022 Antonio Diaz Diaz.
+   Copyright (C) 2013-2024 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -23,10 +23,13 @@ inline bool uid_gid_in_range( const long long uid, const long long gid )
            gid == (long long)( (gid_t)gid ); }
 
 const char * const dotdot_msg = "Contains a '..' component, skipping.";
-const char * const intdir_msg = "Failed to create intermediate directory";
 const char * const cantln_msg = "Can't %slink '%s' to '%s'";
 const char * const mkdir_msg = "Can't create directory";
 const char * const mknod_msg = "Can't create device node";
 const char * const mkfifo_msg = "Can't create FIFO file";
 const char * const uftype_msg = "%s: Unknown file type '%c', skipping.";
 const char * const chown_msg = "Can't change file owner";
+
+mode_t get_umask();
+
+struct Chdir_error {};
