@@ -1,5 +1,5 @@
 /* Tarlz - Archiver with multimember lzip compression
-   Copyright (C) 2013-2023 Antonio Diaz Diaz.
+   Copyright (C) 2013-2024 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -354,7 +354,7 @@ bool Extended::parse( const char * const buf, const int edsize,
       if( stored_crc != computed_crc )
         {
         if( verbosity >= 2 )
-          std::fprintf( stderr, "CRC32C = %08X\n", (unsigned)computed_crc );
+          std::fprintf( stderr, "CRC32-C = %08X\n", (unsigned)computed_crc );
         return false;
         }
       }
